

function [T,Y]=Eulero_mod(t0,tmax,n,y0,f)

%___________________________________________________________________________
%
% Funzione Eulero_mod
% ____________________
%
% Chiamata:   [T,Y]=Eulero_mod(t0,tmax,n,y0,f);
%
% Commento:
%
% La function Eulero_mod utilizza il metodo di Eulero modificato del
% secondo ordine esplicito  per determinare i valori approssimati
% della soluzione di un problema di Cauchy,  nei nodi t(i),
% i=1,2,...,n+1 contenuti in  un intervallo [t0,tmax].
% Il problema (scalare o vettoriale) e' del tipo:
% 
% 
% y'(t)=f(t,y(t))     con t in (t0,tmax]
% y(t0)=yo
%
%   
% dove:
% y(t)=(y1(t),y2(t),...,ym(t))
% f(t,y(t))=(f1(t,y(t)),f2(t,y(t)),...,fm(t,y(t))
% y(t0)=(y1(t0),y2(t0),...,ym(t0))=(yo1,yo2,...,yom)=yo
%
% con m=1 nel caso scalare e m>1 nel caso vettoriale.
%___________________________________________________________________________ 
%
%
% Parametri di input della function:
% ___________________________________
%
% t0,tmax  estremi dell'intervallo di integrazione;
%
%  n       numero di sottointervalli [t(i),t(i+1)], i=1,...,n
%          in cui si desidera suddividere l'intervallo [t0,tmax].    
%
% y0       vettore riga di lunghezza m contenente la condizione iniziale
%          y(t0)=yo=(yo1,yo2,...,yom).
%
% f        vettore colonna di dimensione m  in cui ogni componente e' 
%          una stringa contenente le espressioni f1, f2,..,fm scritte in
%          funzione delle variabili t e y; si puo' creare il vettore f 
%          utilizzando il comando 'strvcat', nel seguente modo: 
%          f=strvcat(f1,f2,...,fm). 
%
%
% Parametri di output della function:
% ___________________________________
%
% T        vettore colonna di dimensione n+1 contenente i nodi t0=t(1),
%          t(2),...,t(n),t(n+1)=tmax.
%
% Y        matrice di dimensione (n+1)x m la cui i-esima riga di lunghezza 
%          m contiene l'approssimazione della funzione y(t)=[y1(t),...,ym(t)], 
%          calcolata nel nodo t(i): y(t(i))=[y1(t(i)),y2(t(i)),...,ym(t(i))].
%___________________________________________________________________________
%
%
% 1° esempio  di utilizzo della function  applicata al seguente 
% problema di Cauchy (caso scalare con m=1):
%  
% y'(t)=y-2t        t e' in (0,1]                       
% y(0)=1.
%
% La soluzione vera del problema e':
% y(t)=-exp(t)+2(t+1).
%
%
%
%      clear all
%      t0=0; tmax=1;
%      y0=1;
%      n=100;
%      f='y-2*t';     
% 
%      [T,Y]= Eulero_mod(t0,tmax,n,y0,f);
%       
%___________________________________________________________________________
%
% 2° esempio  di utilizzo della function  applicata al seguente 
% problema di Cauchy (problema vettoriale del primo ordine con m=2):
%
%
% y1'(t)=0.9-(0.4/450)y1(t)+(0.1/450)y2(t);   t e' in (0,5]   (*)
% y2'(t)=(0.4/450)*y1(t)-(0.4/450)y2(t);
% y1(0)=135
% y2(0)=0
% 
% La soluzione analitica di (*) e':
% y1(t)=-270exp(-t/750)-945exp(-t/2250)+1350;
% y2(t)=540exp(-t/750)-1890exp(-t/2250)+1350;
%
% Osservare che si puo' utilizzare la function anche per risolvere
% numericamente il problema:
% z''(t)=f(t,z',z)
% z(t0)=zo
% z'(t0)=z1
%  
% dopo averlo opportunamente modificato in un problema vettoriale
% del primo ordine.
%
%
% clear all
% t0=0; tmax=5;
% y0=[135 0];
% n=500;
% f=strvcat('0.9-0.4/450*y(1)+0.1/450*y(2)','0.4/450*y(1)-0.4/450*y(2)');
% 
% [T,Y]=Eulero_mod(t0,tmax,n,y0,f);  
%
% % Osservare che in questo caso Y e' una matrice di dimensione 
% % n+1 x 2 contenente nella prima colonna i valori y1(t(i)), 
% % i=1,2,...,n+1, che sono  le approssimazioni nei nodi della 
% % prima componente della soluzione del problema (*); la seconda
% % colonna di Y contiene i valori y2(t(i)) che sono le 
% % approssimazioni della seconda componente della soluzione di (*).
%
%___________________________________________________________________________





h=(tmax-t0)/n;
m=length(y0);
t=t0;y=y0;
T=t0;Y=y0;
y_old=y0;

for i=1:n
   for j=1:m
      k1(j)=eval(f(j,:));
   end
   t=t+h/2; y=y_old+h/2*k1;
   for j=1:m
      k2(j)=eval(f(j,:));
   end
   y=y_old+h*k2;
   t=t+h/2;
   y_old=y;
   T=[T; t];
   Y=[Y; y];
end


